/*Please review License.h tab for use/rights information regarding included libraries
  BMP180_GLCD based on original sketch by M. Ray Burnette 3/25/2014 PUBLIC DOMAIN by Author
  Compiled under Arduino GUI 1.7.9 & Linux 17.3 for 24 hour graphing 20160411
      Sketch uses 27,780 bytes (22%) of program storage space. Maximum is 122,880 bytes.
      Global variables use 3,368 bytes of dynamic memory.
*/

#define BOARD_BUTTON_PIN 32        // definition removed from core so must define here
#include <SPI.h>                   // ~/Arduino/hardware/STM32/STM32F1/libraries/SPI
#include <Wire.h>                  // I2C ~/Arduino/hardware/STM32/STM32F1/libraries/Wire (legacy)
#include <Streaming.h>             // ~/Arduino/libraries/Streaming (legacy)

#include "./BMP085.h"              // #include "I2CDEV.h" is pulled in also
#include "./Adafruit_GFX.h"
#include "./Adafruit_ILI9341.h"

/*  History array fills from left and element [80] is dropped every nn minutes (average calc period)
 0         1         2         3         4         5         6         7         8
 012345678901234567890123456789012345678901234567890123456789012345678901234567890  */
long pHistory[] = {
        0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
        0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 
        0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 
        0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L };
        
const int16_t xRect  =   1;                                       // Block area to erase in History area: x, y, width, height, color
const int16_t yRect  =  88;
const int16_t wRect  = 248;
const int16_t hRect  = 112;
int   localCorrect   = 3575;                                      //  +/- hPa*100 to correct for local altitude: 100X delta:  3625== Briscoe Field, Ga
float temperature;
float pressure;                                                   // How many hPa in 1 mm Hg? The answer is 1.3332239
float inHg;                         
float mmhg_conversion= 3386.388140071641;                         // convert hPa to inches Hg
float altitude;
int32_t lastMicros;
byte error, address  = 77;                                        // I2C address specific to BMP180: MapleMini Chnl1: Data == pin 15, Clock = pin 16
unsigned long TimeMarker;
unsigned long PlotTime;                                           // trigger point to store Barometric reading
long M;                                                           // pressure/100 --> hPa
const unsigned long GraphResponsemS = 1080000L ;                  // 18 minutes per array element,  18*80 = 1440 minutes = 24 hours total
uint8_t increment    = 0;
boolean flip         = true;
float   Celsius      = 0.0;
boolean FirstTime    = true;                                      // Before satellite time is available via $GPRMC or $GPGAA
boolean Fahrenheit   = true;                                      // set by digital pin D12
boolean ForcedUpdate = false;                                     // used to force a time+date screen full redraw

// ILI9341 TFT GLCD display connections for hardware SPI
// Signal           Maple Mini         Leonardo      LCD Display    UNO pins
//#define _sclk         6         //         15       J2 pin 7          13
//#define _miso         5 NC      //         14          pin 9          12
//#define _mosi         4         //         16          pin 6          11
#define TFT_CS         13         //         10          pin 3          10
#define TFT_DC         12         //          9          pin 5           9
#define TFT_RST        14         //          8          pin 4           8

// create lcd object
Adafruit_ILI9341 lcd = Adafruit_ILI9341(TFT_CS, TFT_DC, TFT_RST); // Using hardware SPI

BMP085 barometer;                                                 // create barometer object


void setup()  {
  lcd.begin();
  lcd.setRotation(3);                                             // landscape with SPI conn J2 to left side.
  lcd.fillScreen(ILI9341_BLACK);
  lcd.setCursor(0, 0);
  lcd.setTextColor(ILI9341_WHITE, ILI9341_BLACK);
  // lcd.setTextSize(1);                                          // Tiny   53 char / line
  // lcd.setTextSize(2);                                          // Small  26 char / line
  lcd.setTextSize(3);                                             // Medium 17 char / line  10Pixels Wide x 25 Pixels / line
  //lcd.setTextSize(4);                                           // Large  13 char / line
  lcd.setCursor(0, 0);
  lcd.print("Testing circuit:\n\r\n\r");

  Wire.begin(); barometer.initialize();                           // I2C Stuff
  Wire.beginTransmission(address);                                // I2C Stuff
  error              = Wire.endTransmission();

  if(barometer.testConnection())
  {
    lcd.setTextColor(ILI9341_GREEN, ILI9341_BLACK);
    lcd.print("I2C: OK   V160411");
  } else {
    lcd.print("BMP180: Failure!");
    for(;; ) {;}                                                  // Error with infinite loop
  }

  pinMode(BOARD_BUTTON_PIN, INPUT);                               // board button (for manual History select)
  lcd.drawFastHLine(0, 23, 319, ILI9341_RED);                     // Hello Screen
  lcd.setTextColor(ILI9341_CYAN, ILI9341_BLACK);
  lcd.setCursor(0, 25);
  lcd.print("by: Ray Burnette");
  lcd.setCursor(0, 50);
  lcd.println("Barometric Pres.");
 
  lcd.setTextColor(ILI9341_GREEN, ILI9341_BLACK);
  lcd.setTextSize(1);
  lcd.setTextWrap(false);
  lcd.fillScreen(ILI9341_BLACK);
  lcd.setCursor(30, 100);
  lcd.print( "History updates every 18 minutes");
}


void loop()  {
  
  barometer.setControl(BMP085_MODE_TEMPERATURE);                  // request temperature
  lastMicros         = micros();
  while (micros() - lastMicros < barometer.getMeasureDelayMicroseconds()); // wait for conversion (4.5ms delay)

  if (Fahrenheit) {                                               // read calibrated temperature value in degrees Fahrenheit
    temperature      = barometer.getTemperatureF();
  } else {
    temperature      = barometer.getTemperatureC();
  }

  lastMicros         = micros();
  barometer.setControl(BMP085_MODE_PRESSURE_3);                   // request pressure (3x oversampling mode, high detail, 23.5ms delay)
  while (micros() - lastMicros < barometer.getMeasureDelayMicroseconds());

  pressure           =  barometer.getPressure();                  // read calibrated pressure value in Pascals (Pa)
  pressure          += localCorrect;
  inHg               = float ( pressure * 10.0 / mmhg_conversion);
  M                  = pressure / 100L;                           // scale to hPa
    // graph values are typically between a low of 985 and a high of 1040
    // clipping high/low forces the dynamic range to be limited to a maximum change of 45 hPa
  if (M > 1040L)  M  = 1040L;                                     // clip (flatten) ceiling
  if (M < 985L)   M  =  985L;                                     // clip (flatten) floor

  if ((TimeMarker + GraphResponsemS) < millis() )
  {                                                               // machine clock has advanced beyond trigger, so store current pressure
      TimeMarker     = millis();                                  // reset trigger for future (next) event
      StoreHistory();                                             // update pHistory matrix
      lcd.fillRect(xRect, yRect,wRect, hRect, 0) ;                // erase history graph area with transparancy
      displayHistory();                                           // Intent is to only display on command
  }
  displayCurrent();                                               // Header and current sensor data
  drawLegends();                                                  // Grid and right-hand graph legends
}


void StoreHistory ( void ) {                                      // Creates a boxed border using lines 0 & 1
  int  n;                                                         // avoid compiler complaints in for(;;) loop
  // array [0] is reserved for newest value                       // Therefore [80] = [79], [79] = [78],..., [1]==[0] after shift
  for (n = 80; n > 0; n--)                                        // shift history array to the right, oldest value [80] is lost
  {
    pHistory[n]      = pHistory[(n - 1)];                         // shift data right, oldest value lost, [1] becomes [0] the current value
  }
  pHistory[0]        = M;                                         // save current "adjusted" pressure hPa [0]
}


void displayCurrent( void ) {
  int temp1, temp2;                                               // working vars for type conversion

  lcd.setTextSize(3);                                             // Print hPa and inHg legends and readings in large font
  lcd.setCursor(0, 1);  lcd.print("hPa   inHg   temp");
  lcd.setCursor(0, 35); lcd.print( M );

  if ( M < 1000 ) {
    lcd.print("   ");
  } else {
    lcd.print("  ");
  }

  temp1              = inHg /10;                                  // Integer component
  temp2              = ((inHg / 10) - (float) temp1 ) * 100;      // 2 decimal places
  lcd.print(temp1); lcd.print(".");
  lcd.print(temp2); lcd.print("  ") ;
  lcd.print( temperature, 0 );

  if (Fahrenheit) {
    lcd.print("F");
  } else {
    lcd.print("C");
  }
}


void displayHistory( void ) {
  long     M;
  long     minimum   = 1040L;
  long     maximum   = 985L;

  uint8_t  h, z;
  uint16_t x, y;

  z = 0;                                                          // z will span 0 to 319, full screen landscape width in pixels
  for ( x = 1; x < 240; x++ )  {                                  // 80 elements @1 element per 18 minutes = 10 elements per 3 hours
    if ( x % 3 == 0 ) {                                           // this provides for skipping every 3th position for bar spacing
      z++;
    } else if (pHistory[z] != 0) {                                // array is initially init to zero value, no need to plot this
      if (pHistory[z] < minimum) minimum = pHistory[z];           // capture the lowest 24 hour pressure reading
      if (pHistory[z] > maximum) maximum = pHistory[z];           // capture the highest 24 hour pressure reading
      h              = (pHistory[z] - 985) * 2;                   // h is for height and x2 is for scaling factor
      y              = 200 - h;                                   // Adafruit's GFX y coordinates plots upside down, flip reference
      lcd.drawFastVLine(x,  y,   h, ILI9341_CYAN);                // 2 bars are drawn per array element and 1 blank space for separation
    }
  }

  y                  = 200 - ((minimum - 985) * 2);
  lcd.drawFastHLine(1, y, 245, ILI9341_BLUE);                     // draw 24 hour lowest reading

  y                  = 200 - ((maximum - 985) * 2);
  lcd.drawFastHLine(1, y, 245, ILI9341_YELLOW);                   // draw 24 hour highest reading
}


void drawLegends( void )
{
  // Legends and axis for display
  lcd.setTextSize(1);
  // Atlanta: High = 30.79 on 1/6/1924	Low = 29.08 on 1/11/1918
  // 2X scaling: 1040 - 1026 = 14 x 2 = 28 pixels
  lcd.setCursor(240,  84); lcd.print("_ 1040= 30.71");            // setup Y legends
  lcd.setCursor(240, 112); lcd.print("_ 1026= 30.30");
  lcd.setCursor(240, 140); lcd.print("_ 1012= 29.88");
  lcd.setCursor(240, 168); lcd.print("_  998= 29.47");
  lcd.setCursor(240, 194); lcd.print("_  985= 29.09");
  // tiny fonts for timeline legend @53 char/line (6px/char)
  lcd.setCursor(0, 205); lcd.print("N        ^         ^         ^         ^  Historical ");
  lcd.setCursor(0, 213); lcd.print("O        0         1         1         2  Timeline   ");
  lcd.setCursor(0, 221); lcd.print("W        6         2         8         4  In Hours   ");
  lcd.drawFastVLine(0,  88, 112, ILI9341_RED);                    // setup Y axis
  lcd.drawFastHLine(0, 200, 238, ILI9341_RED);                    // setup X axis
  lcd.drawFastHLine(0, 201, 238, ILI9341_RED);                    // setup X axis double-width X axis for emphasis
  // delay(10000);
}


